import os

from pandas.core.algorithms import value_counts
from jmetal.lab.experiment import Experiment, Job, generate_summary_from_experiment
from jmetal.lab.visualization.plotting import Plot
from jmetal.util.solution import get_non_dominated_solutions
from jmetal.core.quality_indicator import HyperVolume
from jmetal.core.solution import Solution
from matplotlib import pyplot as plt
import matplotlib
import linecache
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42

def find_extrema_onefile(dataFolder, algo: str, benchmark: str, instances: list, n_run: int, folder: str, namefile: str):
    
    for instance in instances:
        size = instance['size']
        name = instance['name']
        id = instance['id']
        print(id, name, size)
        max_obj1 = 0
        min_obj1 = 1000000
        max_obj2 = 0
        min_obj2 = 1000000
        if benchmark == "Solomon":
            subpath = os.path.join('Solomon', 'solomon_'+size, name + id)
            
        elif benchmark == "Generated":
            if int(id) < 10:
                id = "0" + id
            subpath = os.path.join('generated_instances', 'gen_'+size, name + id) 
            
        for run in range(n_run):
            max_obj1_run = 0
            max_obj2_run = 0
            min_obj1_run = 1000000
            min_obj2_run = 1000000
            input_path = os.path.join(dataFolder,algo,subpath,"Run"+str(run+1), folder)
            path_filename = os.path.join(input_path, namefile)
            if path_filename is None:
                raise FileNotFoundError('Filename can not be None')

            with open(path_filename) as file:
                lines = file.readlines()
                data = [line.lstrip() for line in lines if line != ""]
                nbLines = len(data)
                for line in data:
                    int_line = [float(i) for i in line.split()]
                    obj1 = int_line[0]
                    obj2 = int_line[1]
                    if obj1 > max_obj1_run:
                        max_obj1_run = obj1

                    if obj2 > max_obj2_run:
                        max_obj2_run = obj2

                    if obj1 < min_obj1_run:
                        min_obj1_run = obj1

                    if obj2 < min_obj2_run:
                        min_obj2_run = obj2

            if max_obj1_run > max_obj1:
                max_obj1 = max_obj1_run

            if max_obj2_run > max_obj2:
                max_obj2 = max_obj2_run

            if min_obj1_run < min_obj1:
                min_obj1 = min_obj1_run

            if min_obj2_run < min_obj2:
                min_obj2 = min_obj2_run

        output_path = os.path.join(dataFolder, algo, subpath)
        output_file = os.path.join(output_path, 'BEST.tsv')
        with open(output_file, 'w+') as of:
            res_obj1 = str(min_obj1) + " " + str(max_obj1) + " \n"
            res_obj2 = str(min_obj2) + " " + str(max_obj2)
            of.write(res_obj1)
            of.write(res_obj2)

        # update extremas
        extrema_path = os.path.join('extrema', subpath)
        extrema_file = os.path.join(extrema_path, 'extrema.txt')

        if not os.path.exists(extrema_path):
            os.makedirs(extrema_path)
        if not os.path.isfile(extrema_file):
            with open(extrema_file, 'w+') as file:
                 res_obj1 = str(min_obj1) + " " + str(max_obj1) + " \n"
                 res_obj2 = str(min_obj2) + " " + str(max_obj2) + " "
                 file.write(res_obj1)
                 file.write(res_obj2)
            
        with open(extrema_file, 'r') as file:
            lines = file.readlines()
            data = [line.lstrip() for line in lines if line != ""]
            min_objectives = [min_obj1, min_obj2]
            max_objectives = [max_obj1, max_obj2]
            current_objective = 0
            for line in data:
                values = [float(i) for i in line.split()]
                if values[0] < min_objectives[current_objective]: 
                    min_objectives[current_objective] = values[0]
                if values[1] > max_objectives[current_objective]:
                    max_objectives[current_objective] = values[1]
                current_objective += 1
        
        with open(extrema_file, 'w+') as wf:
            res_obj1 = str(min_objectives[0]) + " " + str(max_objectives[0]) + " \n"
            res_obj2 = str(min_objectives[1]) + " " + str(max_objectives[1]) + " "
            wf.write(res_obj1)
            wf.write(res_obj2)


def compute_hypervolumes(dataFolder: str, algo: str, benchmark: str, instances: list, n_run: int, folder: str, filename: str):
    """
    compute all the hypervolumes of PF obtained with algo for (number, type, size) problem
    snapshots is a boolean which indicates if we also compute the hypervolumes for PF conserved during evolution
    """
    for instance in instances:
        size = instance['size']
        name = instance['name']
        id = instance['id']

        if benchmark == "Solomon":
                subpath = os.path.join('Solomon', 'solomon_'+size, name + id)

        elif benchmark == "Generated":
            if int(id) < 10:
                id = "0" + id
            subpath = os.path.join('generated_instances', 'gen_'+size, name + id)
        extrema_path = os.path.join('extrema', subpath, 'extrema.txt')
        nadir_point = []
        ideal_point = []
        with open(extrema_path, 'r') as file:
            lines = file.readlines()
            for line in lines:
                values = [float(i) for i in line.split(' ')[:-1]]
                ideal_point.append(values[0])
                nadir_point.append(values[1])
        reference_point = [1.001, 1.001]
        indicator = HyperVolume(reference_point)
        all_output_path = os.path.join(dataFolder,algo,subpath)
        
        hypervolumes_path = os.path.join(dataFolder, 'HypervolumesOnly', subpath)

        if not os.path.exists(hypervolumes_path):
            os.makedirs(hypervolumes_path)

        hypervolumes_file = os.path.join(hypervolumes_path, 'hypervolumes_'+str(10)+"_"+algo+".tsv") # 10 is final
        bestHV_file = os.path.join(hypervolumes_path, 'bestHV.tsv')
        if os.path.isfile(hypervolumes_file):
            os.remove(hypervolumes_file)

        with open(hypervolumes_file, 'a+') as of:
            of.write("Hypervolume \n")

        all_output_file = os.path.join(all_output_path, 'ALL-HYPERVOLUMES_'+str(10)+'.tsv') # contains all the hypervolumes obtained

        if os.path.isfile(all_output_file):
            os.remove(all_output_file)

        maxHV = 0
        all_hypervolumes = []
        for run in range(n_run):
            solutions = []
            input_path = os.path.join(dataFolder,algo,subpath,"Run"+str(run+1),folder)
            output_path = input_path
            output_file = os.path.join(output_path, 'HYPERVOLUME.tsv')
            mean_output_path = os.path.join(dataFolder,algo,subpath)
            mean_output_file = os.path.join(mean_output_path, 'MEAN-HYPERVOLUME.tsv') # str(6) 

            path_filename = os.path.join(input_path, filename)

            if path_filename is None:
                raise FileNotFoundError('Filename can not be None')

            with open(path_filename) as file:
                lines = file.readlines()
                data = [line.lstrip() for line in lines if line != ""]
                nbLines = len(data)
                for line in data:
                    solution = Solution(int(size), 2)
                    int_line = [float(i) for i in line.split()]
                    obj1 = int_line[0]
                    obj2 = int_line[1]
                    norm_obj1 = round((obj1 - ideal_point[0])/max(1, (nadir_point[0] - ideal_point[0])),5)
                    norm_obj2 = round((obj2 - ideal_point[1])/max(1, (nadir_point[1] - ideal_point[1])),5)
                    solution.objectives[0] = norm_obj1
                    solution.objectives[1] = norm_obj2
                    solutions.append(solution)
            result = indicator.compute([solutions[i].objectives for i in range(len(solutions))])
            if result > maxHV:
                maxHV = round(result, 3)
            all_hypervolumes.append(result)
            #print("Hypervolume of run ", run, ": ", result)
            with open(output_file, 'w+') as of:
                of.write(str(round(result,5)))


            with open(all_output_file, 'a+') as of:
                line = str(round(result,5)) + " \n"
                of.write(line)

            with open(hypervolumes_file, 'a+') as of:
                line = str(round(result,5)) + " \n"
                of.write(line)


        mean_hypervolume = round(sum(all_hypervolumes)/len(all_hypervolumes),5)
        with open(mean_output_file, 'w+') as of:
            of.write(str(mean_hypervolume))


def ecriture_resultats_latex(dataFolder, list_algo: list, list_instances: list, stat: str):
    """ 
    return a file which follows the latex rules for a tabular
    :param list_algo: Names of algorithms that are compared
    :param list_instances: should contain only one type of instance with all sizes
    :param stat: either "hypervolume" or "time"
    """
    if stat != "hypervolume" or stat != "time":
        raise ValueError("stat is hypervolume or time")

    sample_instance = list_instances[0]
    type_instance = sample_instance['name']
    output_path = os.path.join(dataFolder,'tables')
    output_file = os.path.join(output_path, "Results_"+type_instance+".tsv")
    if not os.path.exists(output_path):
        os.makedirs(output_path)
    if os.path.isfile(output_file):
        os.remove(output_file)
    with open(output_file, 'w+') as of:
        line = "Instance" + " & " + "Size" + " & " + "$\mathcal{A}^{LS}$" + " & " +  "$_{all}\mathcal{A}^{LS}_{ML-int}$" + " & " + "$_{all}\mathcal{A}^{LS}_{ML-div}$" + " & " + "$_{lo}\mathcal{A}^{LS}_{ML-int}$" + " & " + "$_{lo}\mathcal{A}^{LS}_{ML-int}$" + "\\\\ \n"
        of.write(line)

    # the user can change the labels according to the classification wanted
    labels = ['1', '2']
    gaps = {}
    for l in labels:
        gaps[l] = {}
        for algo in list_algo:
            gaps[l][algo] = 0 

    list_instances.sort(key = lambda x: (int(x['id']), int(x['size'])))
    for instance in list_instances:
        size = instance['size']
        name_instance = instance['name'] + instance['id']
        allValues = []
        label = instance['id'][0]
        line = ""
        for algo in list_algo:
            input_path = os.path.join(dataFolder, algo, 'Solomon', 'solomon_'+size, name_instance)
            if stat == "hypervolume":
                input_file = os.path.join(input_path, 'MEAN-HYPERVOLUME.tsv')
            elif stat == "time":
                input_file = os.path.join(input_path, 'MEAN-TIME.tsv')
            value = round(float(linecache.getline(input_file, 1).split(' ')[0]),3)
            allValues.append(value)
            if stat == "hypervolume":
                gaps[label][algo] += (value/allValues[0] - 1)*100
            elif stat == "time":
                gaps[label][algo] += (1 - value/allValues[0])*100

        # writing the line of the tabular
        if stat == "hypervolume":
            extremValue = max(allValues)
        elif stat == "time":
            extremValue = min(allValues)

        indexExt = [i for i in range(len(allValues)) if allValues[i] == extremValue]
        line += name_instance + " & " + size
        for i in range(len(list_algo)):
            if i in indexExt:
                line += " & " + "\\textbf{"+str(allValues[i])+"}"
            else:
                line += " & " + str(allValues[i])

        with open(output_file, 'a+') as of:
            line += " \\\\ \n"
            of.write(line)
    
    nbInstances = len(list_instances)//2
    with open(output_file, 'a+') as of:
        for l in labels:
            line = "Mean gain & " 
            for algo in list_algo:
                line += " & " + str(round(gaps[l][algo]/nbInstances, 1))  
            line += " \\\\ \n"
            of.write(line)
        


def print_pareto_front_evolution(algo, number, type, size, run):
    # coords = obj
    all_pareto_fronts = []
    all_labels = []
    reference_points = []
    colors = ['black', 'red', 'purple', 'limegreen']
    input_path = os.path.join('data',algo,'solomon_'+size+'_config_perso',type+number)
    name = 'TRUEBEST.tsv'
    filename = os.path.join(input_path, name)
    with open(filename) as file:
        lines = file.readlines()
        data = [line.lstrip() for line in lines if line != ""]
        line_obj1 = [float(i) for i in data[0].split()]
        line_obj2 = [float(i) for i in data[1].split()]
        min_obj1, max_obj1 = line_obj1[0], line_obj1[1]
        min_obj2, max_obj2 = line_obj2[0], line_obj2[1]
    point = [max_obj1, max_obj2]
    reference_points.append(point)

    for i in range(1,12,3):
        solutions = []
        if i == 10:
            input_path = os.path.join('data',algo,'solomon_'+size+'_config_perso',type+number,"Run"+str(run+1),"Final")
            name = 'FUN.brut.tsv'
            all_labels.append('Final Pareto front')
        else:
            input_path = os.path.join('data',algo,'solomon_'+size+'_config_perso',type+number,"Run"+str(run+1),"Snapshots")
            name = 'SNAPSHOT.FUN.'+str(i)+'.tsv'
            all_labels.append('Pareto front of snapshot ' + str(i))
        filename = os.path.join(input_path, name)
        if filename is None:
            raise FileNotFoundError('Filename can not be None')

        with open(filename) as file:
            lines = file.readlines()
            data = [line.lstrip() for line in lines if line != ""]
            nbLines = len(data)
            for line in data:
                solution = Solution(int(size), 2)
                int_line = [float(i) for i in line.split()]
                obj1 = int_line[0]
                obj2 = int_line[1]
                solution.objectives[0] = obj1
                solution.objectives[1] = obj2
                solutions.append(solution)
        pareto_front = get_non_dominated_solutions(solutions)
        MEDIUM_SIZE = 13
        BIGGER_SIZE = 15
        plt.rc('axes', titlesize=BIGGER_SIZE)     # fontsize of the axes title
        plt.rc('axes', labelsize=BIGGER_SIZE)    # fontsize of the x and y labels
        plt.rc('xtick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
        plt.rc('ytick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
        plt.rc('legend', fontsize=MEDIUM_SIZE)    # legend fontsize
        plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title
        x = [s.objectives[0] for s in pareto_front]
        y = [s.objectives[1] for s in pareto_front]
        plt.scatter(x,y, c = colors[(i-1)//3], label = 'Pareto-front after ' + str(i) + ' minutes')

    #plot = Plot(title = 'Evolution of the Pareto front of instance R204 (of size 100)', axis_labels = ['Total cost','Total waiting time'], reference_point = reference_points)
    #plot.plot(all_pareto_fronts,all_labels)
    plt.xlabel('Total cost')
    plt.ylabel('Total waiting time')
    plt.legend()
    plt.show()

def print_convergence_hypervolume_algos(list_algos, number, type, size):
    x = [1,2,3,4,5,6,7,8,9,10]
    all_y = []
    for algo in list_algos:
        y = []
        for i in range(1,11):
            if i == 10:
                input_path = os.path.join('data',algo,'solomon_'+size,type+number)
                filename = os.path.join(input_path, 'MEAN-HYPERVOLUME.tsv')

            else:
                input_path = os.path.join('data',algo,'solomon_'+size,type+number)
                filename = os.path.join(input_path, 'SNAPSHOT.MEAN-HYPERVOLUME.'+str(i)+'.tsv')

            if filename is None:
                raise FileNotFoundError('Filename can not be None')

            with open(filename) as file:
                lines = file.readlines()
                data = [line.lstrip() for line in lines if line != ""]
                nbLines = len(data)
                for line in data:
                    int_line = [float(i) for i in line.split()]
                    y.append(int_line[0])
        all_y.append(y)
    MEDIUM_SIZE = 13
    BIGGER_SIZE = 15
    plt.rc('axes', titlesize=BIGGER_SIZE)     # fontsize of the axes title
    plt.rc('axes', labelsize=BIGGER_SIZE)    # fontsize of the x and y labels
    plt.rc('xtick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
    plt.rc('ytick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
    plt.rc('legend', fontsize=MEDIUM_SIZE)    # legend fontsize
    plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title
    labels = list_algos
    for i in range(len(list_algos)):
        plt.plot(x,all_y[i], 'o-', label = labels[i])
    plt.xlabel('Time (minutes)')
    plt.ylabel('Mean of the hypervolume')
    plt.legend()
    plt.show()

def print_convergence_hypervolume_instances(algo, benchmark, instances, nbSnapshots):
    x = [i for i in range(2, 2*nbSnapshots+1, 2)]
    all_y = []
    labels = []
    for instance in instances:
        size = instance['size']
        name = instance['name']
        id = instance['id']
        if benchmark == "Solomon":
            subpath = os.path.join('Solomon', 'solomon_'+size, name + id)

        elif benchmark == "Generated":
            if int(id) < 10:
                id = "0" + id
            subpath = os.path.join('generated_instances', 'gen_'+size, name + id)

        y = []
        labels.append(name+id)
        for i in range(1,nbSnapshots+1):
            if i == nbSnapshots:
                input_path = os.path.join('data',algo,subpath)
                filename = os.path.join(input_path, 'MEAN-HYPERVOLUME.tsv')

            else:
                input_path = os.path.join('data',algo,subpath)
                filename = os.path.join(input_path, 'SNAPSHOT.MEAN-HYPERVOLUME.'+str(i)+'.tsv')

            if filename is None:
                raise FileNotFoundError('Filename can not be None')

            with open(filename) as file:
                lines = file.readlines()
                data = [line.lstrip() for line in lines if line != ""]
                nbLines = len(data)
                for line in data:
                    value = [float(i) for i in line.split()]
                    y.append(value[0])
        all_y.append(y)
    MEDIUM_SIZE = 13
    BIGGER_SIZE = 15
    plt.rc('axes', titlesize=BIGGER_SIZE)     # fontsize of the axes title
    plt.rc('axes', labelsize=BIGGER_SIZE)    # fontsize of the x and y labels
    plt.rc('xtick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
    plt.rc('ytick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
    plt.rc('legend', fontsize=MEDIUM_SIZE)    # legend fontsize
    plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title

    for i in range(len(instances)):
        plt.plot(x,all_y[i], 'o-', label = labels[i])
    plt.xlabel('Time (minutes)')
    plt.ylabel('Mean of the hypervolume')
    plt.title(algo)
    #plt.legend()
    plt.show()

def bestFront_algo(algo, instances, benchmark, nbRuns):
    """
    Merge all the fronts obtained by the same algorithm over all the runs
    :param algo: Name of the algorithm
    :param instances: list of instances
    :param benchmark: "Solomon" or "Generated"
    :param nbRuns: Number of runs
    """
    for instance in instances:
        size = instance['size']
        name = instance['name']
        id = instance['id']
        if benchmark == "Solomon":
            subpath = os.path.join('data', algo, 'Solomon', 'solomon_'+size, name + id)

        elif benchmark == "Generated":
            if int(id) < 10:
                id = "0" + id
            subpath = os.path.join('data', algo, 'generated_instances', 'gen_'+size, name + id)

        allSolutions = []
        for i in range(1, nbRuns):
            index = len(allSolutions)
            front_file_FUN = os.path.join(subpath, 'Run'+str(i), 'Final', 'FUN.brut.tsv')
            front_file_VAR = os.path.join(subpath, 'Run'+str(i), 'Final', 'VAR.tsv')
            with open(front_file_FUN, 'r') as file:
                allLines = file.readlines()
                for line in allLines:
                    solution = Solution(int(size), 2)
                    objectives = [float(l) for l in line.split(' ')[:-1]]
                    solution.objectives = objectives.copy()
                    allSolutions.append(solution)
            with open(front_file_VAR, 'r') as file:
                allLines = file.readlines()
                for j in range(len(allLines)):
                    line = allLines[j]
                    variables = [int(k) for k in line.split(' ')[:-1]]
                    allSolutions[j+index].variables = variables.copy()
        front = get_non_dominated_solutions(allSolutions)
        front.sort(key = lambda x: x.objectives[0])
        output_fun = os.path.join(subpath, 'FUN.tsv')
        output_var = os.path.join(subpath, 'VAR.tsv')
        x = []
        y = []
        with open(output_fun, 'w') as file:
            for sol in front:
                line = str(sol.objectives[0]) + ' ' + str(sol.objectives[1])
                x.append(sol.objectives[0])
                y.append(sol.objectives[1])
                file.write(line + '\n')
        with open(output_var, 'w') as file:
            for sol in front:
                line = ""
                for i in sol.variables:
                    line += str(i) + ' '
                file.write(line + '\n')

        MEDIUM_SIZE = 13
        BIGGER_SIZE = 15
        plt.rc('axes', titlesize=BIGGER_SIZE)     # fontsize of the axes title
        plt.rc('axes', labelsize=BIGGER_SIZE)    # fontsize of the x and y labels
        plt.rc('xtick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
        plt.rc('ytick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
        plt.rc('legend', fontsize=MEDIUM_SIZE)    # legend fontsize
        plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title
        plt.figure(figsize=[8, 8])
        plt.scatter(x,y)
        plt.title('Pareto-front of ' + name+id + '-' + size + ' with algorithm ' + algo)
        plt.xlabel('Total traveled distance')
        plt.ylabel('Total waiting time')
        output_figure = os.path.join(subpath, 'front.svg')
        plt.savefig(output_figure)
        plt.close()
    return 

def bestFront_obtained(allAlgos, instances, benchmark):
    """
    Merge all the fronts obtained by all the algorithms specified. 
    """
    for instance in instances:
        size = instance['size']
        name = instance['name']
        id = instance['id']

        allSolutions = []
        for algo in allAlgos:
            index = len(allSolutions)
            if benchmark == "Solomon":
                subpath = os.path.join('Solomon', 'solomon_'+size, name + id)

            elif benchmark == "Generated":
                if int(id) < 10:
                    id = "0" + id
                subpath = os.path.join('generated_instances', 'gen_'+size, name + id)

            front_file_FUN = os.path.join('data', algo, subpath, 'FUN.tsv')
            front_file_VAR = os.path.join('data', algo, subpath, 'VAR.tsv')
            with open(front_file_FUN, 'r') as file:
                for line in file.readlines():
                    solution = Solution(int(size), 2)
                    objectives = [float(i) for i in line.split(' ')]
                    solution.objectives = objectives.copy()
                    allSolutions.append(solution)
            with open(front_file_VAR, 'r') as file:
                allLines = file.readlines()
                for i in range(len(allLines)):
                    line = allLines[i]
                    variables = [int(k) for k in line.split(' ')[:-1]]
                    allSolutions[index+i].variables = variables.copy() 
        front = get_non_dominated_solutions(allSolutions)
        front.sort(key = lambda x: x.objectives[0])
        output_path = os.path.join('bestFronts', subpath)
        
        output_fun = os.path.join(output_path, 'FUN.tsv')
        output_var = os.path.join(output_path, 'VAR.tsv')

        if not os.path.exists(output_path):
            os.makedirs(output_path)
        x = []
        y = []
        with open(output_fun, 'w') as file:
            for sol in front:
                line = str(sol.objectives[0]) + ' ' + str(sol.objectives[1])
                x.append(sol.objectives[0])
                y.append(sol.objectives[1])
                file.write(line + '\n')
        with open(output_var, 'w') as file:
            for sol in front:
                line = ""
                for i in sol.variables:
                    line += str(i) + ' '
                file.write(line + '\n')

        MEDIUM_SIZE = 13
        BIGGER_SIZE = 15
        plt.rc('axes', titlesize=BIGGER_SIZE)     # fontsize of the axes title
        plt.rc('axes', labelsize=BIGGER_SIZE)    # fontsize of the x and y labels
        plt.rc('xtick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
        plt.rc('ytick', labelsize=MEDIUM_SIZE)    # fontsize of the tick labels
        plt.rc('legend', fontsize=MEDIUM_SIZE)    # legend fontsize
        plt.rc('figure', titlesize=BIGGER_SIZE)  # fontsize of the figure title
        plt.figure(figsize=[8, 8])
        plt.scatter(x,y)
        plt.title('Pareto-front of ' + name+id + '-' + size)
        plt.xlabel('Total traveled distance')
        plt.ylabel('Total waiting time')
        output_figure = os.path.join('bestFronts', subpath, 'front.svg')
        plt.savefig(output_figure)
        plt.close()
        
    return

def count_single_front(algo, instance, nRun, objectiveToCount):
    """
    Count how many times has algorithm "algo" reached the value objectiveToCount for the instance given.
    :param algo: The name of the algorithm
    :param instance: The characteristics of the instance
    :param nRun: Number of runs
    :param objectiveToCount: The value you are interesting for
    """
    size = instance['size']
    name = instance['name']
    id = instance['id']
    subpath = os.path.join('data_time', algo, 'Solomon', 'solomon_'+size, name + id)
    cpt = 0
    for run in range(1, nRun+1):
        fun_file = os.path.join(subpath, "Run"+str(run), 'Final', 'FUN.brut.tsv')
        with open(fun_file, 'r') as file:
            lines = file.readlines()
            for line in lines:
                obj = float(line.split(' ')[0])
                if obj == objectiveToCount:
                    cpt += 1
    return cpt


def compute_meanTime(dataFolder, algo, instance, nRun):
    """
    Compute the mean time of execution of an algorithm over nRun for an instance.
    """
    size = instance['size']
    name = instance['name']
    id = instance['id']
    subpath = os.path.join(dataFolder, algo, 'Solomon', 'solomon_'+size, name + id)
    totalTime = 0
    for run in range(1, nRun + 1):
        stats_file = os.path.join(subpath, "Run"+str(run), 'Final', 'STATS')
        time = float(linecache.getline(stats_file, 1).split(' ')[-1])
        totalTime += time
    meanTime = round(totalTime/nRun, 2)
    output_file = os.path.join(subpath, 'MEAN-TIME.tsv')
    with open(output_file, 'w') as file:
        file.write(str(meanTime) + '\n')
    return
            
benchmark = "Solomon"
nbSnapshots = 10

if benchmark == "Generated":
    generated_instances = []
    for l1 in range(1, 9):
        for l2 in ['L', 'S']:
            for l3 in ['T', 'W']:
                for l4 in ['R']:
                    instance = {}
                    instance['size'] = '100'
                    instance['name'] = l4 + l2 + l3
                    instance['id'] = str(l1)
                    generated_instances.append(instance)

elif benchmark == "Solomon":
    numbers_instances_C = ['101', '102', '103', '104', '105', '106', '107', '108', '109', '201', '202', '203', '204', '205', '206', '207', '208']
    numbers_instances_R = ['101', '102', '103', '104', '105', '106', '107', '108', '109', '110', '111', '112', '201', '202', '203', '204', '205', '206', '207', '208', '209', '210', '211']
    numbers_instances_RC = ['101', '102', '103', '104', '105', '106', '107', '108', '201', '202', '203', '204', '205', '206', '207', '208']
    number_instances = {}
    number_instances['C'] = numbers_instances_C
    number_instances['R'] = numbers_instances_R
    number_instances['RC'] = numbers_instances_RC

    generated_instances = []
    for size in ['50', '100']:
        for type in ['R', 'C', 'RC']:
            for id in number_instances[type]:
                instance = {}
                instance['size'] = size
                instance['name'] = type
                instance['id'] = id
                generated_instances.append(instance)

n_run = 30
list_algos = ['MOEAD-LS', 'A-int', 'A-div', 'all-ALS-int', 'all-ALS-div', 'lo-ALS-int', 'lo-ALS-div']
dataFolder = "data"
resultFolder = 'Final'
namefile_brut = 'FUN.brut.tsv'

size_spec = {}
size_spec['50'] = 'MEAN-HYPERVOLUME.tsv'
size_spec['100'] = 'MEAN-HYPERVOLUME.tsv'

stat = "time" 
updateExtrema = True
writeToLatex = True
computeReferenceFronts = True

if updateExtrema:
    for algo in list_algos:
        find_extrema_onefile(dataFolder, algo, benchmark, generated_instances, n_run, resultFolder, namefile_brut)


for algo in list_algos:
    if stat == "hypervolume":
        compute_hypervolumes(dataFolder, algo, benchmark, generated_instances, n_run, resultFolder, namefile_brut)
    elif stat == "time":
        compute_meanTime(dataFolder, algo, instance, n_run)

if writeToLatex:
    ecriture_resultats_latex(list_algos,  generated_instances, "time")

#print_pareto_front_evolution(list_algos[0], "101", "RC", "50", 2)
#print_convergence_hypervolume_algos(list_algos, "101", "C", test_sizes[0])
#for algo in list_algos:
#    print_convergence_hypervolume_instances(algo, benchmark, generated_instances, nbSnapshots)

if computeReferenceFronts:
    for algo in list_algos:
        bestFront_algo(algo, generated_instances, benchmark, n_run)
    bestFront_obtained(list_algos, generated_instances, benchmark)
