import copy
import random
from typing import List

from jmetal.core.operator import Extraction
from jmetal.core.solution import Solution,  PermutationSolution
from jmetal.util.ckecking import Check

"""
.. module:: extraction
   :platform: Unix, Windows
   :synopsis: Module implementing crossover operators.

.. moduleauthor:: Clément Legrand <clement.legrand4.etu@univ-lille.fr>
"""

class MOPatternExtraction(Extraction[PermutationSolution, list]):
    def __init__(self, probability):
        super(MOPatternExtraction, self).__init__(probability)

    def extract_fixed_size_patterns_from_one_solution(self, solution: PermutationSolution, size_of_pattern: int) -> list:
        """
        Extract patterns of a fixed size from a solution. 
        """

        list_of_tours = solution.structure
        all_patterns = []
        for tour in list_of_tours:
            patterns = []
            length = len(tour)
            for i in range(1,length-size_of_pattern): # do not consider patterns which contain the depot
                pattern = tour[i:i+size_of_pattern]
                patterns.append((size_of_pattern,pattern))
            all_patterns += patterns
        
        return all_patterns

    def execute(self, solution:PermutationSolution, max_size_of_patterns: int) -> list:
        if random.random() > self.probability:
            return []
        
        all_patterns = []
        for size_of_pattern in range(2, max_size_of_patterns+1):
            patterns = self.extract_fixed_size_patterns_from_one_solution(solution, size_of_pattern)
            all_patterns += patterns
        return all_patterns

    def get_name(self):
        return 'Multi-Objective pattern extraction'
