#!/bin/bash

#SBATCH --partition=2x24
#SBATCH --job-name=hostname
#SBATCH --output=slurm.out
#SBATCH --error=slurm.err
#SBATCH --nodes=1
#SBATCH --ntasks-per-node=1

############
# Options: #
############
# * -d : diversification strategy
# * -i : intensification strategy
# * -l : learning on local optima only
# * -k : display the knowledge generated
# * -h : add the hypervolume as termination criteria (see experiments.py file for more information)

##############
# Arguments: #
##############

# $1 is the name of the algorithm
# $2 is the benchmark used
# $3 is the number of the run
# $4 is the number of the instance
# $5 is the type of instance (see folder resources/VRPTW_instances to find the possible letters)
# $6 is the size of the instance
# $7 is the termination criteria

#############################################
# Arguments to specify from left to right #
#############################################

# populationSize
# sizeNeighborhood
# granularity
# probabilityCrossover
# probabilityMutation
# maxPatternSize
# patternsConsidered
# patternsInjected
# frequencyExtraction
# frequencyInjection

#############################################

# Variants:
# * MOEAD-LS: MOEAD with a local search (abreviated in A)
# * As: MOEAD + LS + strategy s for the injection
# * loAs: MOEAD + LS + strategy s for the injection and extraction only on Local Optima

# This file contains the parameters related to the experiment 1 of the paper.

case $1 in

    loAi)
	case $6 in
	    50)
		python3 experiments.py -i -l $1 1 $2 $3 $4 $5 $6 $7 94 73 16 0.94 0.71 4 60 15 0.60 0.80;;
	    100)
		python3 experiments.py -i -l $1 1 $2 $3 $4 $5 $6 $7 68 55 67 0.99 0.86 5 150 45 0.40 0.90;;
	esac;;
	
    loAd)
	case $6 in
	    50)
		python3 experiments.py -d -l $1 1 $2 $3 $4 $5 $6 $7 94 73 16 0.94 0.71 4 60 15 0.60 0.80;;
	    100)
		python3 experiments.py -d -l $1 1 $2 $3 $4 $5 $6 $7 68 55 67 0.99 0.86 5 150 45 0.40 0.90;;
	esac;;

    Ai)
	case $6 in
	    50)
		python3 experiments.py -i $1 1 $2 $3 $4 $5 $6 $7 94 73 16 0.94 0.71 4 60 15 0.60 0.80;;
	    100)
		python3 experiments.py -i $1 1 $2 $3 $4 $5 $6 $7 68 55 67 0.99 0.86 5 150 45 0.40 0.90;;
	esac;;
	
    Ad)
	case $6 in
	    50)
		python3 experiments.py -d $1 1 $2 $3 $4 $5 $6 $7 94 73 16 0.94 0.71 4 60 15 0.60 0.80;;
	    100)
		python3 experiments.py -d $1 1 $2 $3 $4 $5 $6 $7 68 55 67 0.99 0.86 5 150 45 0.40 0.90;;
	esac;;
    
    MOEAD-LS)
	case $6 in
	    50)
		python3 experiments.py $1 1 $2 $3 $4 $5 $6 $7 94 73 16 0.94 0.71 0 0 0 0 0;;
	    100)
		python3 experiments.py $1 1 $2 $3 $4 $5 $6 $7 68 55 67 0.99 0.86 0 0 0 0 0;;
	esac;;

	    
esac

# example:
# To run the 1st experiment with the algorithm Ai on the instance C101 of size 50 from Solomon Benchmark with a time limit of 300s, write the following command: 
# sh parameters_experiment1.sh Ai Solomon 0 101 C 50 300  


