#!/bin/bash

#SBATCH --partition=2x24
#SBATCH --job-name=hostname
#SBATCH --output=slurm.out
#SBATCH --error=slurm.err
#SBATCH --nodes=1
#SBATCH --ntasks-per-node=1

############
# Options: #
############
# * -d : diversification strategy
# * -i : intensification strategy
# * -l : learning on local optima only
# * -k : display the knowledge generated
# * -h : add the hypervolume as termination criteria (see experiments.py file for more information)

##############
# Arguments: #
##############

# $1 is the name of the algorithm
# $2 is the benchmark used
# $3 is the number of the run
# $4 is the number of the instance
# $5 is the type of instance (see folder resources/VRPTW_instances to find the possible letters)
# $6 is the size of the instance
# $7 is the termination criteria

#############################################
# Arguments to specify from left to right #
#############################################

# populationSize
# sizeNeighborhood
# granularity
# probabilityCrossover
# probabilityMutation
# maxPatternSize
# patternsConsidered
# patternsInjected
# frequencyExtraction
# frequencyInjection

#############################################

# Variants:
# * MOEAD-LS: MOEAD with a local search (abreviated in A)
# * As: MOEAD + LS + strategy s for the injection
# * loAs: MOEAD + LS + strategy s for the injection and extraction only on Local Optima

# This file contains the parameters related to the experiment 2 of the paper.

case $1 in

    loAi)
	case $6 in
	    50)
		python3 experiments.py -i -l $1 1 $2 $3 $4 $5 $6 $7 23 7 21 0.97 0.55 5 86 18 0.45 0.86;;
	    100)
		python3 experiments.py -i -l $1 1 $2 $3 $4 $5 $6 $7 21 12 73 0.78 0.6 6 171 30 0.20 0.97;;
	esac;;
	
    loAd)
	case $6 in
	    50)
		python3 experiments.py -d -l $1 1 $2 $3 $4 $5 $6 $7 31 8 25 0.88 0.31 5 74 17 0.48 0.83;;
	    100)
		python3 experiments.py -d -l $1 1 $2 $3 $4 $5 $6 $7 22 7 87 0.86 0.52 5 135 34 0.26 0.93;;
	esac;;

    Ai)
	case $6 in
	    50)
		python3 experiments.py -i $1 1 $2 $3 $4 $5 $6 $7 50 15 24 0.84 0.49 4 132 48 0.47 0.85;;
	    100)
		python3 experiments.py -i $1 1 $2 $3 $4 $5 $6 $7 52 6 66 0.61 0.48 6 180 70 0.68 0.91;;
	esac;;
	
    Ad)
	case $6 in
	    50)
		python3 experiments.py -d $1 1 $2 $3 $4 $5 $6 $7 61 15 28 0.76 0.54 4 124 32 0.46 0.92;;
	    100)
		python3 experiments.py -d $1 1 $2 $3 $4 $5 $6 $7 60 5 60 0.25 0.30 5 150 60 0.80 0.97;;
	esac;;
    
    MOEAD-LS)
	case $6 in
	    50)
		python3 experiments.py $1 1 $2 $3 $4 $5 $6 $7 94 73 16 0.94 0.71 0 0 0 0 0;;
	    100)
		python3 experiments.py $1 1 $2 $3 $4 $5 $6 $7 68 55 67 0.99 0.86 0 0 0 0 0;;
	esac;;
	    
esac

# example:
# To run the 2nd experiment with the algorithm Ai on the instance C101 of size 50 from Solomon Benchmark with a time limit of 300s, write the following command: 

# sh parameters_experiment2.sh Ai Solomon 0 101 C 50 300  


